/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __DB_MANAGER_H__
#define __DB_MANAGER_H__

#include <string>
#include <sqlite3.h>
#include <vector>

#define CREATE_TAC_DB_TABLE "CREATE TABLE IF NOT EXISTS TAC(\
			ID INTEGER PRIMARY KEY AUTOINCREMENT,\
			PKGID		TEXT,\
			NUGET		TEXT,\
			NAME		TEXT,\
			VERSION		TEXT);"
#define QUERY_MAX_LEN	4096

sqlite3* dbCreate(std::string path);
sqlite3* dbOpen(std::string path);
void dbClose(sqlite3 *tac_db);
void dbRollback(sqlite3 *tac_db);
bool dbUpdate(sqlite3 *tac_db, std::string path, std::string query);
bool dbInsert(sqlite3 *tac_db, std::string path, std::string query);
std::vector<std::string> dbSelect(sqlite3 *tac_db, std::string path, std::string query);
bool dbDelete(sqlite3 *tac_db, std::string path, std::string query);

#endif /* __DB_MANAGER_H__ */
