/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "log.h"

#include <cstdlib>
#include <cstring>

#include <dlfcn.h>

#define HT_PATH "/home/owner/share/tmp/sdk_tools/heaptrack/"
#define HT_LIB_PATH HT_PATH "libprofiler.so"
#define HT_INJECTION_LIB_PATH HT_PATH "libheaptrack_inject.so"

static int injectLibrary(const char path[])
{
	typedef int inject_func();

	int res = -1;
	void *lib = nullptr;
	inject_func *inject = nullptr;
	const char *inject_sym = "dotnet_launcher_inject";

	_INFO("Inject %s library", path);

	// Current implementation of heaptrack CLR profiler requires RTLD_GLOBAL
	lib = dlopen(path, RTLD_NOW | RTLD_GLOBAL);
	if (lib == nullptr) {
		_ERR("%s", dlerror());
		return res;
	}

	inject = reinterpret_cast<inject_func*>(dlsym(lib, inject_sym));
	if (inject == nullptr) {
		_ERR("%s is not found in the %s", inject_sym, path);
		return res;
	}

	res = inject();
	return res;
}

int checkInjection()
{
	char *env = nullptr;

	env = getenv("CORECLR_PROFILER_PATH");
	if (env == nullptr) {
		return 0;
	}

	// At the moment, this mechanism is used only when the Memory Profiler is started.
	if (strcmp(env, HT_LIB_PATH) != 0) {
		return 0;
	}

	_INFO("##### Perform injection #########");

	if (injectLibrary(HT_INJECTION_LIB_PATH) != 0) {
		_INFO("##### Injection failed #########");
		return -1;
	}

	_INFO("##### Injection finished #########");
	return 0;
}
