/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <unistd.h>
#include <pthread.h>

#include "log.h"
#include "log_manager.h"

static int __pfd[2];
static pthread_t loggingThread;
static bool __hasException = false;


static void *stdlog(void*)
{
    ssize_t readSize;
    char buf[1024];

    while ((readSize = read(__pfd[0], buf, sizeof buf - 1)) > 0) {
        if (buf[readSize - 1] == '\n') {
            --readSize;
        }

        buf[readSize] = 0;

		if (strstr(buf, "Unhandled Exception:") != NULL) {
			__hasException = true;
		}

        _LOGX("%s", buf);
    }

	close(__pfd[0]);
	close(__pfd[1]);

    return 0;
}

int initializeLogManager()
{
    if (setvbuf(stdout, NULL, _IOLBF, 0) < 0) {
		_ERR("fail to make stdout line-buffered");
		return -1;
    }

    if (setvbuf(stderr, NULL, _IONBF, 0) < 0) {
		_ERR("make stderr unbuffered");
		return -1;
	}

    /* create the pipe and redirect stdout and stderr */
    if (pipe(__pfd) < 0) {
		_ERR("fail to create pipe for logging");
		return -1;
    }

	return 0;
}

int redirectFD()
{
	if (__pfd[1] < 0) {
		_ERR("fail to create pipe for logging");
		return -1;
	}
    // stdout
    if (dup2(__pfd[1], 1) == -1) {
		_ERR("fail to duplicate fd to stdout");
		return -1;
    }

    // stderr
    if (dup2(__pfd[1], 2) == -1) {
		_ERR("fail to duplicate fd to stderr");
		return-1;
	}

	return 0;
}

int runLoggingThread()
{
    /* spawn the logging thread */
    if (pthread_create(&loggingThread, 0, stdlog, 0) != 0) {
		_ERR("fail to create pthread");
        return -1;
    }

    if (pthread_detach(loggingThread) != 0) {
		_ERR("fail to detach pthread");
		return -1;
	}

    return 0;
}

int hasException()
{
	return __hasException;
}
